/**
 *****************************************************************************
   @example  ADT7x40_Demo.c
   @brief    
	 The ADT7410 is connected to I2C configured in master mode
         - SCL on P2.0
         - SDA on P2.1
	 - A0 and A1 of the temperature sensor should be connected to GND.
	 The ADT7310 is connected to SPI0 configured in master mode
   - MISO on P1.4
	 - SCLK on P1.5
	 - MOSI on P1.6
	 - CS on P1.7
   Temperature measurements are sent back via UART to the PC
	 - Rx on P0.1
	 - Tx on P0.2

   @version  V0.1
   @author   ADI
   @date     July 2013 
              

All files for ADuCM360/61 provided by ADI, including this file, are
provided  as is without warranty of any kind, either expressed or implied.
The user assumes any and all risk from the use of this code.
It is the responsibility of the person integrating this code into an application
to ensure that the resulting application performs as required and is safe.

**/

#include <stdio.h>
#include <string.h>
#include "ADuCM360.h"
#include "ADT7410I2C.h"
#include "ADT7310SPI.h"
#include <..\common\ClkLib.h>
#include <..\common\I2cLib.h>
#include <..\common\SpiLib.h>
#include <..\common\UrtLib.h>
#include <..\common\DioLib.h>
#include <..\common\WdtLib.h>

void UARTINIT(void);                  // Configures and enables UART
void SPI0SETUP(void);                 // Configures and enables SPI0
void I2CSETUP(void);                  // Configures and enables I2C
void SendString(void);								// Transmit a string using UART
void SendResultToUART(void);          // Transmit temperature sensors results
void delay(int);                      // Software delay


unsigned char ucTxBufferEmpty  = 0;       // Used to indicate that the UART Tx buffer is empty
unsigned char szTemp[64] = "";            // Used to store string before printing to UART
unsigned char nLen = 0;                   // String length

const int bit = 13;

unsigned char I2CStatus = 0;
unsigned char I2CConfig = 0;
volatile float I2Ctemp = 0.0;
volatile float SPItemp = 0.0;

unsigned char I2C_ID = 0; 
unsigned char SPI_ID = 0; 


int main(void)
{
  WdtGo(T3CON_ENABLE_DIS); 
	
  // Configure ADuCM360 clocks
  ClkCfg(CLK_CD0,CLK_HF,CLKSYSDIV_DIV2EN_DIS,CLK_UCLKCG);
  ClkSel(CLK_CD0,CLK_CD0,CLK_CD0,CLK_CD0);
  ClkDis(0|CLKDIS_DISPWMCLK|CLKDIS_DIST0CLK|CLKDIS_DIST1CLK|CLKDIS_DISDMACLK|CLKDIS_DISADCCLK);  

  // Uart,I2C and SPI initalization on ADuCM360
  UARTINIT();
  I2CSETUP();
  SPI0SETUP();

  // Enable Uart,I2C and SPI Interrupts 
  NVIC_EnableIRQ(I2CM_IRQn);
  NVIC_EnableIRQ(UART_IRQn);
  NVIC_EnableIRQ(SPI0_IRQn);

  // Configure ADT7410_I2C and ADT7310_SPI
  ADT7410_I2CCFG(ADT7410_ADDRESS,BIT16,ADT7410_OP_MODE_CONT_CONV,Active_Low,Active_Low,interrupt_m,ADT7410_FAULT_QUEUE_1_FAULT);
  ADT7310_SPICFG(Bit_16,cont_conv,CT_pin_L,INT_pin_L,interupt_mode,fault_1);

  // Reading TEMPERATURE SENSOR ID 
  I2C_ID = ADT7410_I2CID(ADT7410_ADDRESS);
  SPI_ID = ADT7310_SPI_Status(REG_ID);

  while (1)
  {
    I2Ctemp = ADT7410_I2CGetTemperature(ADT7410_ADDRESS,&I2CStatus,&I2CConfig);
    SPItemp = ADT7310_SPIGetTemperature();

    SendResultToUART();   

    delay(0xFFFF);   // a timer could be used here for more precise scheduling of the temperature readings
    delay(0xFFFF);     
  } 
}

void UARTINIT (void)
{
   DioCfg(pADI_GP0,0x3C);	                               // Configure p0.1/p0.2 as UART pins
   UrtCfg(pADI_UART,B9600,COMLCR_WLS_8BITS,0);            // setup baud rate for 9600, 8-bits
   UrtMod(pADI_UART,COMMCR_DTR,0);  			             // Setup modem bits
   UrtIntCfg(pADI_UART,COMIEN_ERBFI|COMIEN_ETBEI|COMIEN_ELSI|COMIEN_EDSSI|COMIEN_EDMAT|COMIEN_EDMAR);  // Setup UART IRQ sources
}

void I2CSETUP (void)
{
  DioCfg(pADI_GP2,0x05);  // Configure P2.0/P2.1 as I2C pins
  DioPul(pADI_GP2,0xFF); // Disable Internal pull-ups on I2C pins. external pull ups should be used */
   
  /* Enable I2C Master mode, baud rate and interrupt sources */
  I2cMCfg(I2CMCON_TXDMA_DIS|I2CMCON_RXDMA_DIS,I2CMCON_IENCMP|SPICON_RXOF_EN|I2CMCON_IENRX|I2CMCON_IENTX,I2CMCON_MAS_EN);      
  I2cBaud(0x1E,0x1F);   // 100kHz clock
}

void SPI0SETUP(void)
{
  DioCfg(pADI_GP1,0xAA00);     // Configure P1.4/p1.5/p1.6/p1.7 as SPI0 pins  

  /* Enable SPI Master mode, baud rate and interrupt sources */
  SpiCfg(pADI_SPI0,SPICON_MOD_TX2RX2,SPICON_MASEN_EN,SPICON_CON_EN|SPICON_TIM_TXWR|SPICON_CPHA_SAMPLETRAILING|SPICON_CPOL_HIGH|SPICON_ENABLE_EN);
  SpiBaud(pADI_SPI0,63,SPIDIV_BCRST_DIS);    // 125kHz clock 
}


void SendString (void)
{
  unsigned char i = 0;
	
  for ( i = 0 ; i < nLen ; i++ )	// loop to send ADC0 result
  {
    ucTxBufferEmpty = 0;
    UrtTx(pADI_UART,szTemp[i]);
    while (ucTxBufferEmpty == 0)
    {
    }
  }
} 

void SendResultToUART(void)
{  
  sprintf ( (char*)szTemp, "ADT7310 ID: %d \r\n ",SPI_ID );                           
  nLen = strlen((char*)szTemp);  
  if (nLen <64)   
    SendString(); 
  sprintf ( (char*)szTemp, "ADT7310 Temperature: %fC \r\n ",SPItemp );                            
  nLen = strlen((char*)szTemp);  
  if (nLen <64)  
    SendString();  
  sprintf ( (char*)szTemp, "ADT7410 ID: %d \r\n ",I2C_ID );                            
  nLen = strlen((char*)szTemp);  
  if (nLen <64)  
    SendString(); 
  sprintf ( (char*)szTemp, "ADT7410 Temperature: %fC \r\n\n ",I2Ctemp );                           
  nLen = strlen((char*)szTemp);  
  if (nLen <64)  
    SendString();  
  if(I2CConfig &(1<<7))
  { 
    sprintf ( (char*)szTemp, "ADT7410 BIT Resolution: %d \r\n\n ",bit+3);                          
    nLen = strlen((char*)szTemp);
    if (nLen <64)
      SendString();  
  }
  else 
  {
    sprintf ( (char*)szTemp, "ADT7410 BIT Resolution: %d \r\n ",bit);                          
    nLen = strlen((char*)szTemp);
    if (nLen <64)
      SendString();
  } 
}

void UART_Int_Handler ()
{
  volatile unsigned char ucCOMIID0 = 0;

  ucCOMIID0 = UrtIntSta(pADI_UART);   // Read UART Interrupt ID register
  if ((ucCOMIID0 & COMIIR_STA_TXBUFEMPTY) == COMIIR_STA_TXBUFEMPTY)       // Transmit buffer empty
  {
    ucTxBufferEmpty = 1;
  }
  if ((ucCOMIID0 & 0x4) == 0x4)       // Receive byte
  {
  }
} 
